"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const request_utils_1 = require("./request-utils");
const mymonero_config_1 = require("../mymonero-config");
const xmr_constants_1 = require("@xmr-core/xmr-constants");
const response_parsers_1 = require("./response-parsers");
const biginteger_1 = require("@xmr-core/biginteger");
const xmr_crypto_utils_1 = require("@xmr-core/xmr-crypto-utils");
const xmr_str_utils_1 = require("@xmr-core/xmr-str-utils");
const errors_1 = require("@xmr-core/xmr-mymonero-libs/src/mymonero-send-tx/internal_libs/errors");
class MyMoneroApi {
    static async login(address, privViewKey) {
        const parameters = {
            address,
            view_key: privViewKey,
            create_account: true,
        };
        const data = await request_utils_1.makeRequest(mymonero_config_1.myMoneroConfig.hostName, "login", parameters);
        return data.new_address;
    }
    static async addressInfo(address, privViewKey, pubSpendKey, privSpendKey, hwdev) {
        const parameters = {
            address,
            view_key: privViewKey,
        };
        const data = await request_utils_1.makeRequest(mymonero_config_1.myMoneroConfig.hostName, "get_address_info", parameters);
        return response_parsers_1.parseAddressInfo(address, data, xmr_crypto_utils_1.isRealDevice(hwdev)
            ? (await hwdev.get_secret_keys()).viewKey
            : privViewKey, pubSpendKey, privSpendKey, hwdev);
    }
    static async addressTransactions(address, privViewKey, pubSpendKey, privSpendKey, hwdev) {
        const parameters = {
            address,
            view_key: privViewKey,
        };
        const data = await request_utils_1.makeRequest(mymonero_config_1.myMoneroConfig.hostName, "get_address_txs", parameters);
        return response_parsers_1.parseAddressTransactions(address, data, xmr_crypto_utils_1.isRealDevice(hwdev)
            ? (await hwdev.get_secret_keys()).viewKey
            : privViewKey, pubSpendKey, privSpendKey, hwdev);
    }
    // Getting wallet txs import info
    static async importRequestInfoAndStatus(address, privViewKey) {
        const parameters = request_utils_1.withUserAgentParams({
            address,
            view_key: privViewKey,
        });
        const data = await request_utils_1.makeRequest(mymonero_config_1.myMoneroConfig.hostName, "import_wallet_request", parameters);
        return {
            payment_id: data.payment_id,
            payment_address: data.payment_address,
            import_fee: new biginteger_1.BigInt(data.import_fee),
            feeReceiptStatus: data.feeReceiptStatus,
        };
    }
    // Getting outputs for sending funds
    static async unspentOutputs(address, privViewKey, pubSpendKey, privSpendKey, mixinNumber, hwdev) {
        xmr_str_utils_1.JSONPrettyPrint("unspentOutputs", {
            address,
            privViewKey,
            pubSpendKey,
            privSpendKey,
            mixinNumber,
        }, "args");
        const parameters = request_utils_1.withUserAgentParams({
            address,
            view_key: privViewKey,
            amount: "0",
            mixin: mixinNumber,
            use_dust: true,
            dust_threshold: xmr_constants_1.config.dustThreshold.toString(),
        });
        const data = await request_utils_1.makeRequest(mymonero_config_1.myMoneroConfig.hostName, "get_unspent_outs", parameters);
        xmr_str_utils_1.JSONPrettyPrint("unspentOutputs", {
            parameters,
            data,
        }, "pre_parseUnspentOutputs_dataAndParams");
        return response_parsers_1.parseUnspentOutputs(address, data, xmr_crypto_utils_1.isRealDevice(hwdev)
            ? (await hwdev.get_secret_keys()).viewKey
            : privViewKey, pubSpendKey, privSpendKey, hwdev);
    }
    static async randomOutputs(usingOuts, mixin) {
        xmr_str_utils_1.JSONPrettyPrint("randomOutputs", { usingOuts, mixin }, "args");
        if (mixin < 0 || isNaN(mixin)) {
            throw Error("Invalid mixin - must be >= 0");
        }
        const amounts = usingOuts.map(o => (o.rct ? "0" : o.amount.toString()));
        const parameters = request_utils_1.withUserAgentParams({
            amounts,
            count: mixin + 1,
        });
        const data = await request_utils_1.makeRequest(mymonero_config_1.myMoneroConfig.hostName, "get_random_outs", parameters);
        xmr_str_utils_1.JSONPrettyPrint("randomOutputs", { amount_outs: data.amount_outs }, "ret");
        return { amount_outs: data.amount_outs };
    }
    static async submitSerializedSignedTransaction(address, privViewKey, serializedSignedTx) {
        const parameters = request_utils_1.withUserAgentParams({
            address,
            view_key: privViewKey,
            tx: serializedSignedTx,
        });
        try {
            return await request_utils_1.makeRequest(mymonero_config_1.myMoneroConfig.hostName, "submit_raw_tx", parameters);
        }
        catch (e) {
            console.error(e);
            throw errors_1.ERR.TX.submitUnknown(e);
        }
    }
}
exports.MyMoneroApi = MyMoneroApi;
//# sourceMappingURL=host-api.js.map